@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# Copyright 2011-2017, Damian Johnson and The Tor Project
+# Copyright 2011-2018, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -161,6 +161,7 @@ def main():
     test.task.STEM_VERSION,
     test.task.TOR_VERSION if args.run_integ else None,
     test.task.PYTHON_VERSION,
+    test.task.PLATFORM_VERSION,
     test.task.CRYPTO_VERSION,
     test.task.PYNACL_VERSION,
     test.task.MOCK_VERSION,
@@ -235,9 +236,15 @@ def main():
         println('Running tests...\n', STATUS)
 
         for test_class in get_integ_tests(args.specific_test):
-          run_result = _run_test(args, test_class, output_filters)
-          test.output.print_logging(logging_buffer)
-          skipped_tests += len(getattr(run_result, 'skipped', []))
+          if integ_runner.assert_tor_is_running():
+            run_result = _run_test(args, test_class, output_filters)
+            test.output.print_logging(logging_buffer)
+            skipped_tests += len(getattr(run_result, 'skipped', []))
+          else:
+            # our tor process died
+
+            error_tracker.register_error()
+            break
       except KeyboardInterrupt:
         println('  aborted starting tor: keyboard interrupt\n', ERROR)
         break
