@@ -1,4 +1,4 @@
-# Copyright 2011-2017, Damian Johnson and The Tor Project
+# Copyright 2011-2018, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -272,7 +272,7 @@ def connect(control_port = ('127.0.0.1', 'default'), control_socket = '/var/run/
       if port == 'default':
         control_connection = _connection_for_default_port(address)
       else:
-        control_connection = stem.socket.ControlPort(address, port)
+        control_connection = stem.socket.ControlPort(address, int(port))
     except stem.SocketError as exc:
       error_msg = CONNECT_MESSAGES['unable_to_use_port'].format(address = address, port = port, error = exc)
 
@@ -375,7 +375,7 @@ def _connect_auth(control_socket, password, password_prompt, chroot_path, contro
       return controller(control_socket, is_authenticated = True)
   except IncorrectSocketType:
     if isinstance(control_socket, stem.socket.ControlPort):
-      print(CONNECT_MESSAGES['wrong_port_type'].format(port = control_socket.get_port()))
+      print(CONNECT_MESSAGES['wrong_port_type'].format(port = control_socket.port))
     else:
       print(CONNECT_MESSAGES['wrong_socket_type'])
 
