@@ -1,4 +1,4 @@
-# Copyright 2012-2017, Damian Johnson and The Tor Project
+# Copyright 2012-2018, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -64,7 +64,6 @@ import stem.util.tor_tools
 import stem.version
 
 from stem.descriptor.router_status_entry import RouterStatusEntryV3
-from stem.util import str_type
 
 from stem.descriptor import (
   PGP_BLOCK_END,
@@ -90,10 +89,9 @@ from stem.descriptor import (
   _random_crypto_blob,
 )
 
-try:
-  # added in python 3.2
+if stem.prereq._is_lru_cache_available():
   from functools import lru_cache
-except ImportError:
+else:
   from stem.util.lru_cache import lru_cache
 
 # relay descriptors must have exactly one of the following
@@ -338,15 +336,6 @@ def _parse_hibernating_line(descriptor, entries):
   descriptor.hibernating = value == '1'
 
 
-def _parse_hidden_service_dir_line(descriptor, entries):
-  value = _value('hidden-service-dir', entries)
-
-  if value:
-    descriptor.hidden_service_dir = value.split(' ')
-  else:
-    descriptor.hidden_service_dir = ['2']
-
-
 def _parse_uptime_line(descriptor, entries):
   # We need to be tolerant of negative uptimes to accommodate a past tor
   # bug...
@@ -421,7 +410,7 @@ def _parse_history_line(keyword, history_end_attribute, history_interval_attribu
 
 def _parse_exit_policy(descriptor, entries):
   if hasattr(descriptor, '_unparsed_exit_policy'):
-    if descriptor._unparsed_exit_policy == [str_type('reject *:*')]:
+    if descriptor._unparsed_exit_policy and stem.util.str_tools._to_unicode(descriptor._unparsed_exit_policy[0]) == 'reject *:*':
       descriptor.exit_policy = REJECT_ALL_POLICY
     else:
       descriptor.exit_policy = stem.exit_policy.ExitPolicy(*descriptor._unparsed_exit_policy)
@@ -449,6 +438,7 @@ _parse_ipv6_policy_line = _parse_simple_line('ipv6-policy', 'exit_policy_v6', fu
 _parse_allow_single_hop_exits_line = _parse_if_present('allow-single-hop-exits', 'allow_single_hop_exits')
 _parse_tunneled_dir_server_line = _parse_if_present('tunnelled-dir-server', 'allow_tunneled_dir_requests')
 _parse_proto_line = _parse_protocol_line('proto', 'protocols')
+_parse_hidden_service_dir_line = _parse_if_present('hidden-service-dir', 'is_hidden_service_dir')
 _parse_caches_extra_info_line = _parse_if_present('caches-extra-info', 'extra_info_cache')
 _parse_bridge_distribution_request_line = _parse_simple_line('bridge-distribution-request', 'bridge_distribution')
 _parse_family_line = _parse_simple_line('family', 'family', func = lambda v: set(v.split(' ')))
@@ -494,6 +484,8 @@ class ServerDescriptor(Descriptor):
 
   :var list link_protocols: link protocols supported by the relay
   :var list circuit_protocols: circuit protocols supported by the relay
+  :var bool is_hidden_service_dir: **\*** indicates if the relay serves hidden
+    service descriptors
   :var bool hibernating: **\*** hibernating when published
   :var bool allow_single_hop_exits: **\*** flag if single hop exiting is allowed
   :var bool allow_tunneled_dir_requests: **\*** flag if tunneled directory
@@ -527,6 +519,13 @@ class ServerDescriptor(Descriptor):
   .. versionchanged:: 1.6.0
      Added the extra_info_sha256_digest, protocols, and bridge_distribution
      attributes.
+
+  .. versionchanged:: 1.7.0
+     Added the is_hidden_service_dir attribute.
+
+  .. versionchanged:: 1.7.0
+     Deprecated the hidden_service_dir field, it's never been populated
+     (:spec:`43c2f78`). This field will be removed in Stem 2.0.
   """
 
   ATTRIBUTES = {
@@ -555,6 +554,7 @@ class ServerDescriptor(Descriptor):
 
     'link_protocols': (None, _parse_protocols_line),
     'circuit_protocols': (None, _parse_protocols_line),
+    'is_hidden_service_dir': (False, _parse_hidden_service_dir_line),
     'hibernating': (False, _parse_hibernating_line),
     'allow_single_hop_exits': (False, _parse_allow_single_hop_exits_line),
     'allow_tunneled_dir_requests': (False, _parse_tunneled_dir_server_line),
@@ -562,7 +562,6 @@ class ServerDescriptor(Descriptor):
     'extra_info_cache': (False, _parse_caches_extra_info_line),
     'extra_info_digest': (None, _parse_extrainfo_digest_line),
     'extra_info_sha256_digest': (None, _parse_extrainfo_digest_line),
-    'hidden_service_dir': (None, _parse_hidden_service_dir_line),
     'eventdns': (None, _parse_eventdns_line),
     'ntor_onion_key': (None, _parse_ntor_onion_key_line),
     'or_addresses': ([], _parse_or_address_line),
@@ -633,6 +632,12 @@ class ServerDescriptor(Descriptor):
 
     entries, self._unparsed_exit_policy = _descriptor_components(stem.util.str_tools._to_unicode(raw_contents), validate, extra_keywords = ('accept', 'reject'), non_ascii_fields = ('contact', 'platform'))
 
+    # TODO: Remove the following field in Stem 2.0. It has never been populated...
+    #
+    #   https://gitweb.torproject.org/torspec.git/commit/?id=43c2f78
+
+    self.hidden_service_dir = ['2']
+
     if validate:
       self._parse(entries, validate)
 
@@ -829,7 +834,7 @@ class RelayDescriptor(ServerDescriptor):
         if signed_digest != self.digest():
           raise ValueError('Decrypted digest does not match local digest (calculated: %s, local: %s)' % (signed_digest, self.digest()))
 
-        if self.onion_key_crosscert:
+        if self.onion_key_crosscert and stem.prereq.is_crypto_available():
           onion_key_crosscert_digest = self._digest_for_signature(self.onion_key, self.onion_key_crosscert)
 
           if onion_key_crosscert_digest != self._onion_key_crosscert_digest():
