@@ -1,4 +1,4 @@
-# Copyright 2012-2016, Damian Johnson and The Tor Project
+# Copyright 2012-2018, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -67,20 +67,19 @@ exiting to a destination is permissible or not. For instance...
 
 from __future__ import absolute_import
 
+import re
 import socket
 import zlib
 
 import stem.prereq
+import stem.util
 import stem.util.connection
 import stem.util.enum
 import stem.util.str_tools
 
-from stem.util import _hash_attr, str_type
-
-try:
-  # added in python 3.2
+if stem.prereq._is_lru_cache_available():
   from functools import lru_cache
-except ImportError:
+else:
   from stem.util.lru_cache import lru_cache
 
 AddressType = stem.util.enum.Enum(('WILDCARD', 'Wildcard'), ('IPv4', 'IPv4'), ('IPv6', 'IPv6'))
@@ -109,6 +108,12 @@ def get_config_policy(rules, ip_address = None):
   * ports being optional
   * the 'private' keyword
 
+  .. deprecated:: 1.7.0
+
+     Tor's torrc parameters lack a formal spec, making it difficult for this
+     method to be reliable. Callers are encouraged to move to
+     :func:`~stem.control.Controller.get_exit_policy` instead.
+
   :param str,list rules: comma separated rules or list to be converted
   :param str ip_address: this relay's IP address for the 'private' policy if
     it's present, this defaults to the local address
@@ -123,7 +128,7 @@ def get_config_policy(rules, ip_address = None):
   elif ip_address and stem.util.connection.is_valid_ipv6_address(ip_address, allow_brackets = True) and not (ip_address[0] == '[' and ip_address[-1] == ']'):
     ip_address = '[%s]' % ip_address  # ExitPolicy validation expects IPv6 addresses to be bracketed
 
-  if isinstance(rules, (bytes, str_type)):
+  if stem.util._is_str(rules):
     rules = rules.split(',')
 
   result = []
@@ -134,7 +139,7 @@ def get_config_policy(rules, ip_address = None):
     if not rule:
       continue
 
-    if ':' not in rule:
+    if not re.search(':[\d\-\*]+$', rule):
       rule = '%s:*' % rule
 
     if 'private' in rule:
@@ -237,7 +242,7 @@ class ExitPolicy(object):
     # sanity check the types
 
     for rule in rules:
-      if not isinstance(rule, (bytes, str_type, ExitPolicyRule)):
+      if not stem.util._is_str(rule) and not isinstance(rule, ExitPolicyRule):
         raise TypeError('Exit policy rules can only contain strings or ExitPolicyRules, got a %s (%s)' % (type(rule), rules))
 
     # Unparsed representation of the rules we were constructed with. Our
@@ -248,7 +253,7 @@ class ExitPolicy(object):
     is_all_str = True
 
     for rule in rules:
-      if not isinstance(rule, (bytes, str_type)):
+      if not stem.util._is_str(rule):
         is_all_str = False
 
     if rules and is_all_str:
@@ -281,6 +286,9 @@ class ExitPolicy(object):
     :returns: **True** if exiting to this destination is allowed, **False** otherwise
     """
 
+    if not self.is_exiting_allowed():
+      return False
+
     for rule in self._get_rules():
       if rule.is_match(address, port, strict):
         return rule.is_accept
@@ -457,7 +465,7 @@ class ExitPolicy(object):
         if isinstance(rule, bytes):
           rule = stem.util.str_tools._to_unicode(rule)
 
-        if isinstance(rule, str_type):
+        if stem.util._is_str(rule):
           if not rule.strip():
             continue
 
@@ -647,6 +655,8 @@ class ExitPolicyRule(object):
     # policy ::= "accept[6]" exitpattern | "reject[6]" exitpattern
     # exitpattern ::= addrspec ":" portspec
 
+    rule = stem.util.str_tools._to_unicode(rule)
+
     self.is_accept = rule.startswith('accept')
     is_ipv6_only = rule.startswith('accept6') or rule.startswith('reject6')
 
@@ -662,7 +672,7 @@ class ExitPolicyRule(object):
 
     exitpattern = exitpattern.lstrip()
 
-    if ':' not in exitpattern:
+    if ':' not in exitpattern or ']' in exitpattern.rsplit(':', 1)[1]:
       raise ValueError("An exitpattern must be of the form 'addrspec:portspec': %s" % rule)
 
     self.address = None
@@ -906,7 +916,7 @@ class ExitPolicyRule(object):
   def _get_mask_bin(self):
     # provides an integer representation of our mask
 
-    return int(stem.util.connection._get_address_binary(self.get_mask(False)), 2)
+    return int(stem.util.connection._address_to_binary(self.get_mask(False)), 2)
 
   @lru_cache()
   def _get_address_bin(self):
@@ -983,7 +993,7 @@ class ExitPolicyRule(object):
       else:
         raise ValueError("The '%s' isn't a number of bits: %s" % (addr_extra, rule))
     else:
-      raise ValueError("Address isn't a wildcard, IPv4, or IPv6 address: %s" % rule)
+      raise ValueError("'%s' isn't a wildcard, IPv4, or IPv6 address: %s" % (addrspec, rule))
 
   def _apply_portspec(self, rule, portspec):
     # Parses the portspec...
@@ -1018,7 +1028,7 @@ class ExitPolicyRule(object):
 
   def __hash__(self):
     if self._hash is None:
-      self._hash = _hash_attr(self, 'is_accept', 'address', 'min_port', 'max_port') * 1024 + hash(self.get_mask(False))
+      self._hash = stem.util._hash_attr(self, 'is_accept', 'address', 'min_port', 'max_port') * 1024 + hash(self.get_mask(False))
 
     return self._hash
 
@@ -1047,7 +1057,6 @@ class MicroExitPolicyRule(ExitPolicyRule):
     self.address = None  # wildcard address
     self.min_port = min_port
     self.max_port = max_port
-    self._hash = None
     self._skip_rule = False
 
   def is_address_wildcard(self):
@@ -1063,10 +1072,7 @@ class MicroExitPolicyRule(ExitPolicyRule):
     return None
 
   def __hash__(self):
-    if self._hash is None:
-      self._hash = _hash_attr(self, 'is_accept', 'min_port', 'max_port')
-
-    return self._hash
+    return stem.util._hash_attr(self, 'is_accept', 'min_port', 'max_port', cache = True)
 
   def __eq__(self, other):
     return hash(self) == hash(other) if isinstance(other, MicroExitPolicyRule) else False
