@@ -1,4 +1,4 @@
-# Copyright 2015-2017, Damian Johnson and The Tor Project
+# Copyright 2015-2018, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -54,23 +54,21 @@ import sys
 import tempfile
 
 import stem.prereq
+import stem.util
 import stem.util.conf
 import stem.util.enum
 import stem.util.log
 import stem.util.system
 
-from stem.util import _hash_attr
-
 try:
   # added in python 2.7
   from collections import OrderedDict
 except ImportError:
   from stem.util.ordereddict import OrderedDict
 
-try:
-  # added in python 3.2
+if stem.prereq._is_lru_cache_available():
   from functools import lru_cache
-except ImportError:
+else:
   from stem.util.lru_cache import lru_cache
 
 try:
@@ -79,9 +77,9 @@ try:
 except ImportError:
   import urllib2 as urllib
 
-Category = stem.util.enum.Enum('GENERAL', 'CLIENT', 'RELAY', 'DIRECTORY', 'AUTHORITY', 'HIDDEN_SERVICE', 'TESTING', 'UNKNOWN')
+Category = stem.util.enum.Enum('GENERAL', 'CLIENT', 'RELAY', 'DIRECTORY', 'AUTHORITY', 'HIDDEN_SERVICE', 'DENIAL_OF_SERVICE', 'TESTING', 'UNKNOWN')
 GITWEB_MANUAL_URL = 'https://gitweb.torproject.org/tor.git/plain/doc/tor.1.txt'
-CACHE_PATH = os.path.join(os.path.dirname(__file__), 'cached_tor_manual.sqlite')
+CACHE_PATH = os.path.join(os.path.dirname(__file__), 'cached_manual.sqlite')
 DATABASE = None  # cache database connections
 HAS_ENCODING_ARG = not stem.util.system.is_mac() and not stem.util.system.is_bsd() and not stem.util.system.is_slackware()
 
@@ -104,6 +102,7 @@ CATEGORY_SECTIONS = OrderedDict((
   ('DIRECTORY SERVER OPTIONS', Category.DIRECTORY),
   ('DIRECTORY AUTHORITY SERVER OPTIONS', Category.AUTHORITY),
   ('HIDDEN SERVICE OPTIONS', Category.HIDDEN_SERVICE),
+  ('DENIAL OF SERVICE MITIGATION OPTIONS', Category.DENIAL_OF_SERVICE),
   ('TESTING NETWORK OPTIONS', Category.TESTING),
 ))
 
@@ -130,6 +129,12 @@ def query(query, *param):
   be treated as being read-only. File permissions generally enforce this, and
   in the future will be enforced by this function as well.
 
+  ::
+
+    >>> import stem.manual
+    >>> print(stem.manual.query('SELECT description FROM torrc WHERE key=?', 'CONTROLSOCKET').fetchone()[0])
+    Like ControlPort, but listens on a Unix domain socket, rather than a TCP socket.  0 disables ControlSocket. (Unix and Unix-like systems only.) (Default: 0)
+
   .. versionadded:: 1.6.0
 
   :param str query: query to run on the cache
@@ -184,7 +189,7 @@ class ConfigOption(object):
     self.description = description
 
   def __hash__(self):
-    return _hash_attr(self, 'name', 'category', 'usage', 'summary', 'description')
+    return stem.util._hash_attr(self, 'name', 'category', 'usage', 'summary', 'description', cache = True)
 
   def __eq__(self, other):
     return hash(self) == hash(other) if isinstance(other, ConfigOption) else False
@@ -352,11 +357,11 @@ class Manual(object):
   :var str synopsis: brief tor command usage
   :var str description: general description of what tor does
 
-  :var dict commandline_options: mapping of commandline arguments to their descripton
-  :var dict signals: mapping of signals tor accepts to their description
-  :var dict files: mapping of file paths to their description
+  :var collections.OrderedDict commandline_options: mapping of commandline arguments to their descripton
+  :var collections.OrderedDict signals: mapping of signals tor accepts to their description
+  :var collections.OrderedDict files: mapping of file paths to their description
 
-  :var dict config_options: :class:`~stem.manual.ConfigOption` tuples for tor configuration options
+  :var collections.OrderedDict config_options: :class:`~stem.manual.ConfigOption` tuples for tor configuration options
 
   :var str man_commit: latest tor commit editing the man page when this
     information was cached
@@ -367,10 +372,10 @@ class Manual(object):
     self.name = name
     self.synopsis = synopsis
     self.description = description
-    self.commandline_options = commandline_options
-    self.signals = signals
-    self.files = files
-    self.config_options = config_options
+    self.commandline_options = OrderedDict(commandline_options)
+    self.signals = OrderedDict(signals)
+    self.files = OrderedDict(files)
+    self.config_options = OrderedDict(config_options)
     self.man_commit = None
     self.stem_commit = None
     self.schema = None
@@ -472,9 +477,9 @@ class Manual(object):
       conf.get('name', ''),
       conf.get('synopsis', ''),
       conf.get('description', ''),
-      conf.get('commandline_options', {}),
-      conf.get('signals', {}),
-      conf.get('files', {}),
+      conf.get('commandline_options', OrderedDict()),
+      conf.get('signals', OrderedDict()),
+      conf.get('files', OrderedDict()),
       config_options,
     )
 
@@ -647,7 +652,7 @@ class Manual(object):
     conf.save(path)
 
   def __hash__(self):
-    return _hash_attr(self, 'name', 'synopsis', 'description', 'commandline_options', 'signals', 'files', 'config_options')
+    return stem.util._hash_attr(self, 'name', 'synopsis', 'description', 'commandline_options', 'signals', 'files', 'config_options', cache = True)
 
   def __eq__(self, other):
     return hash(self) == hash(other) if isinstance(other, Manual) else False
@@ -751,8 +756,19 @@ def _add_config_options(config_options, category, lines):
 
   last_option, usage, description = None, None, []
 
-  if lines and lines[0].startswith('The following options'):
-    lines = lines[lines.index(''):]  # drop the initial description
+  # Drop the section description. Each ends with a paragraph saying 'The
+  # following options...'.
+
+  desc_paragraph_index = None
+
+  for i, line in enumerate(lines):
+    if 'The following options' in line:
+      desc_paragraph_index = i
+      break
+
+  if desc_paragraph_index is not None:
+    lines = lines[desc_paragraph_index:]  # trim to the description paragrah
+    lines = lines[lines.index(''):]  # drop the paragraph
 
   for line in lines:
     if line and not line.startswith(' '):
