@@ -1,4 +1,4 @@
-# Copyright 2012-2017, Damian Johnson and The Tor Project
+# Copyright 2012-2018, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -16,9 +16,7 @@ Parses replies from the control socket.
     |- from_str - provides a ControlMessage for the given string
     |- is_ok - response had a 250 status
     |- content - provides the parsed message content
-    |- raw_content - unparsed socket data
-    |- __str__ - content stripped of protocol formatting
-    +- __iter__ - ControlLine entries for the content of the message
+    +- raw_content - unparsed socket data
 
   ControlLine - String subclass with methods for parsing controller responses.
     |- remainder - provides the unparsed content
@@ -36,6 +34,7 @@ import re
 import threading
 
 import stem.socket
+import stem.util
 import stem.util.str_tools
 
 __all__ = [
@@ -129,6 +128,9 @@ class ControlMessage(object):
   Message from the control socket. This is iterable and can be stringified for
   individual message components stripped of protocol formatting. Messages are
   never empty.
+
+  .. versionchanged:: 1.7.0
+     Implemented equality and hashing.
   """
 
   @staticmethod
@@ -170,6 +172,7 @@ class ControlMessage(object):
     self._parsed_content = parsed_content
     self._raw_content = raw_content
     self._str = None
+    self._hash = stem.util._hash_attr(self, '_raw_content')
 
   def is_ok(self):
     """
@@ -299,6 +302,15 @@ class ControlMessage(object):
 
     return ControlLine(content)
 
+  def __hash__(self):
+    return self._hash
+
+  def __eq__(self, other):
+    return hash(self) == hash(other) if isinstance(other, ControlMessage) else False
+
+  def __ne__(self, other):
+    return not self == other
+
 
 class ControlLine(str):
   """
