@@ -1,4 +1,4 @@
-# Copyright 2012-2017, Damian Johnson and The Tor Project
+# Copyright 2012-2018, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 import io
@@ -8,10 +8,12 @@ import time
 import stem
 import stem.control
 import stem.descriptor.router_status_entry
+import stem.prereq
 import stem.response
+import stem.util
 import stem.version
 
-from stem.util import str_type, int_type, connection, log, str_tools, tor_tools
+from stem.util import connection, log, str_tools, tor_tools
 
 # Matches keyword=value arguments. This can't be a simple "(.*)=(.*)" pattern
 # because some positional arguments, like circuit paths, can have an equal
@@ -22,6 +24,10 @@ QUOTED_KW_ARG = re.compile('^(.*) ([A-Za-z0-9_]+)="(.*)"$')
 CELL_TYPE = re.compile('^[a-z0-9_]+$')
 PARSE_NEWCONSENSUS_EVENTS = True
 
+# TODO: We can remove the following when we drop python2.6 support.
+
+INT_TYPE = int if stem.prereq.is_python_3() else long
+
 
 class Event(stem.response.ControlMessage):
   """
@@ -65,6 +71,9 @@ class Event(stem.response.ControlMessage):
 
     self._parse()
 
+  def __hash__(self):
+    return stem.util._hash_attr(self, 'arrived_at', parent = stem.response.ControlMessage, cache = True)
+
   def _parse_standard_attr(self):
     """
     Most events are of the form...
@@ -161,7 +170,7 @@ class Event(stem.response.ControlMessage):
     attr_values = getattr(self, attr)
 
     if attr_values:
-      if isinstance(attr_values, (bytes, str_type)):
+      if stem.util._is_str(attr_values):
         attr_values = [attr_values]
 
       for value in attr_values:
@@ -268,8 +277,8 @@ class BandwidthEvent(Event):
   The BW event was one of the first Control Protocol V1 events and was
   introduced in tor version 0.1.1.1-alpha.
 
-  :var long read: bytes received by tor that second
-  :var long written: bytes sent by tor that second
+  :var int read: bytes received by tor that second
+  :var int written: bytes sent by tor that second
   """
 
   _POSITIONAL_ARGS = ('read', 'written')
@@ -282,8 +291,8 @@ class BandwidthEvent(Event):
     elif not self.read.isdigit() or not self.written.isdigit():
       raise stem.ProtocolError("A BW event's bytes sent and received should be a positive numeric value, received: %s" % self)
 
-    self.read = int_type(self.read)
-    self.written = int_type(self.written)
+    self.read = INT_TYPE(self.read)
+    self.written = INT_TYPE(self.written)
 
 
 class BuildTimeoutSetEvent(Event):
@@ -404,41 +413,15 @@ class CircuitEvent(Event):
     self._log_if_unrecognized('remote_reason', stem.CircClosureReason)
 
   def _compare(self, other, method):
+    # sorting circuit events by their identifier
+
     if not isinstance(other, CircuitEvent):
       return False
 
-    for attr in ('id', 'status', 'path', 'build_flags', 'purpose', 'hs_state', 'rend_query', 'created', 'reason', 'remote_reason', 'socks_username', 'socks_password'):
-      my_attr = getattr(self, attr)
-      other_attr = getattr(other, attr)
+    my_id = getattr(self, 'id')
+    their_id = getattr(other, 'id')
 
-      # Our id attribute is technically a string, but Tor conventionally uses
-      # ints. Attempt to handle as ints if that's the case so we get numeric
-      # ordering.
-
-      if attr == 'id' and my_attr and other_attr:
-        if my_attr.isdigit() and other_attr.isdigit():
-          my_attr = int(my_attr)
-          other_attr = int(other_attr)
-
-      if my_attr is None:
-        my_attr = ''
-
-      if other_attr is None:
-        other_attr = ''
-
-      if my_attr != other_attr:
-        return method(my_attr, other_attr)
-
-    return True
-
-  def __hash__(self):
-    return hash(str(self).strip())
-
-  def __eq__(self, other):
-    return self._compare(other, lambda s, o: s == o)
-
-  def __ne__(self, other):
-    return not self == other
+    return method(my_id, their_id) if my_id != their_id else method(hash(self), hash(other))
 
   def __gt__(self, other):
     return self._compare(other, lambda s, o: s > o)
@@ -564,15 +547,26 @@ class ConfChangedEvent(Event):
 
   The CONF_CHANGED event was introduced in tor version 0.2.3.3-alpha.
 
-  :var dict config: mapping of configuration options to their new values
-    (**None** if the option is being unset)
+  .. deprecated:: 1.7.0
+     Deprecated the *config* attribute. Some tor configuration options (like
+     ExitPolicy) can have multiple values, so a simple 'str => str' mapping
+     meant that we only provided the last.
+
+  .. versionchanged:: 1.7.0
+     Added the changed and unset attributes.
+
+  :var dict changed: mapping of configuration options to a list of their new
+    values
+  :var list unset: configuration options that have been unset
   """
 
   _SKIP_PARSING = True
   _VERSION_ADDED = stem.version.Requirement.EVENT_CONF_CHANGED
 
   def _parse(self):
-    self.config = {}
+    self.changed = {}
+    self.unset = []
+    self.config = {}  # TODO: remove in stem 2.0
 
     # Skip first and last line since they're the header and footer. For
     # instance...
@@ -586,8 +580,10 @@ class ConfChangedEvent(Event):
     for line in str(self).splitlines()[1:-1]:
       if '=' in line:
         key, value = line.split('=', 1)
+        self.changed.setdefault(key, []).append(value)
       else:
         key, value = line, None
+        self.unset.append(key)
 
       self.config[key] = value
 
@@ -652,6 +648,9 @@ class HSDescEvent(Event):
   .. versionchanged:: 1.5.0
      Added the replica attribute.
 
+  .. versionchanged:: 1.7.0
+     Added the index attribute.
+
   :var stem.HSDescAction action: what is happening with the descriptor
   :var str address: hidden service address
   :var stem.HSAuth authentication: service's authentication method
@@ -661,11 +660,12 @@ class HSDescEvent(Event):
   :var str descriptor_id: descriptor identifier
   :var stem.HSDescReason reason: reason the descriptor failed to be fetched
   :var int replica: replica number the descriptor involves
+  :var str index: computed index of the HSDir the descriptor was uploaded to or fetched from
   """
 
   _VERSION_ADDED = stem.version.Requirement.EVENT_HS_DESC
   _POSITIONAL_ARGS = ('action', 'address', 'authentication', 'directory', 'descriptor_id')
-  _KEYWORD_ARGS = {'REASON': 'reason', 'REPLICA': 'replica'}
+  _KEYWORD_ARGS = {'REASON': 'reason', 'REPLICA': 'replica', 'HSDIR_INDEX': 'index'}
 
   def _parse(self):
     self.directory_fingerprint = None
@@ -1084,8 +1084,8 @@ class StreamBwEvent(Event):
      Added the time attribute.
 
   :var str id: stream identifier
-  :var long written: bytes sent by the application
-  :var long read: bytes received by the application
+  :var int written: bytes sent by the application
+  :var int read: bytes received by the application
   :var datetime time: time when the measurement was recorded
   """
 
@@ -1102,8 +1102,8 @@ class StreamBwEvent(Event):
     elif not self.read.isdigit() or not self.written.isdigit():
       raise stem.ProtocolError("A STREAM_BW event's bytes sent and received should be a positive numeric value, received: %s" % self)
 
-    self.read = int_type(self.read)
-    self.written = int_type(self.written)
+    self.read = INT_TYPE(self.read)
+    self.written = INT_TYPE(self.written)
     self.time = self._iso_timestamp(self.time)
 
 
@@ -1154,8 +1154,8 @@ class ConnectionBandwidthEvent(Event):
 
   :var str id: connection identifier
   :var stem.ConnectionType conn_type: connection type
-  :var long read: bytes received by tor that second
-  :var long written: bytes sent by tor that second
+  :var int read: bytes received by tor that second
+  :var int written: bytes sent by tor that second
   """
 
   _KEYWORD_ARGS = {
@@ -1181,8 +1181,8 @@ class ConnectionBandwidthEvent(Event):
     elif not tor_tools.is_valid_connection_id(self.id):
       raise stem.ProtocolError("Connection IDs must be one to sixteen alphanumeric characters, got '%s': %s" % (self.id, self))
 
-    self.read = int_type(self.read)
-    self.written = int_type(self.written)
+    self.read = INT_TYPE(self.read)
+    self.written = INT_TYPE(self.written)
 
     self._log_if_unrecognized('conn_type', stem.ConnectionType)
 
@@ -1199,9 +1199,17 @@ class CircuitBandwidthEvent(Event):
   .. versionchanged:: 1.6.0
      Added the time attribute.
 
+  .. versionchanged:: 1.7.0
+     Added the delivered_read, delivered_written, overhead_read, and
+     overhead_written attributes.
+
   :var str id: circuit identifier
-  :var long read: bytes received by tor that second
-  :var long written: bytes sent by tor that second
+  :var int read: bytes received by tor that second
+  :var int written: bytes sent by tor that second
+  :var int delivered_read: user payload received by tor that second
+  :var int delivered_written: user payload sent by tor that second
+  :var int overhead_read: padding so read cells will have a fixed length
+  :var int overhead_written: padding so written cells will have a fixed length
   :var datetime time: time when the measurement was recorded
   """
 
@@ -1209,6 +1217,10 @@ class CircuitBandwidthEvent(Event):
     'ID': 'id',
     'READ': 'read',
     'WRITTEN': 'written',
+    'DELIVERED_READ': 'delivered_read',
+    'DELIVERED_WRITTEN': 'delivered_written',
+    'OVERHEAD_READ': 'overhead_read',
+    'OVERHEAD_WRITTEN': 'overhead_written',
     'TIME': 'time',
   }
 
@@ -1221,15 +1233,29 @@ class CircuitBandwidthEvent(Event):
       raise stem.ProtocolError('CIRC_BW event is missing its read value')
     elif not self.written:
       raise stem.ProtocolError('CIRC_BW event is missing its written value')
-    elif not self.read.isdigit() or not self.written.isdigit():
-      raise stem.ProtocolError("A CIRC_BW event's bytes sent and received should be a positive numeric value, received: %s" % self)
+    elif not self.read.isdigit():
+      raise stem.ProtocolError("A CIRC_BW event's bytes received should be a positive numeric value, received: %s" % self)
+    elif not self.written.isdigit():
+      raise stem.ProtocolError("A CIRC_BW event's bytes sent should be a positive numeric value, received: %s" % self)
+    elif self.delivered_read and not self.delivered_read.isdigit():
+      raise stem.ProtocolError("A CIRC_BW event's delivered bytes received should be a positive numeric value, received: %s" % self)
+    elif self.delivered_written and not self.delivered_written.isdigit():
+      raise stem.ProtocolError("A CIRC_BW event's delivered bytes sent should be a positive numeric value, received: %s" % self)
+    elif self.overhead_read and not self.overhead_read.isdigit():
+      raise stem.ProtocolError("A CIRC_BW event's overhead bytes received should be a positive numeric value, received: %s" % self)
+    elif self.overhead_written and not self.overhead_written.isdigit():
+      raise stem.ProtocolError("A CIRC_BW event's overhead bytes sent should be a positive numeric value, received: %s" % self)
     elif not tor_tools.is_valid_circuit_id(self.id):
       raise stem.ProtocolError("Circuit IDs must be one to sixteen alphanumeric characters, got '%s': %s" % (self.id, self))
 
-    self.read = int_type(self.read)
-    self.written = int_type(self.written)
     self.time = self._iso_timestamp(self.time)
 
+    for attr in ('read', 'written', 'delivered_read', 'delivered_written', 'overhead_read', 'overhead_written'):
+      value = getattr(self, attr)
+
+      if value:
+        setattr(self, attr, INT_TYPE(value))
+
 
 class CellStatsEvent(Event):
   """
