@@ -270,45 +270,12 @@ class TestController(unittest.TestCase):
   @test.require.controller
   def test_get_exit_policy(self):
     """
-    Sanity test for get_exit_policy(). We have the default policy (no
-    ExitPolicy set) which is a little... long due to the boilerplate.
+    Sanity test for get_exit_policy(). Our 'ExitRelay 0' torrc entry causes us
+    to have a simple reject-all policy.
     """
 
-    expected = ExitPolicy(
-      'reject 0.0.0.0/8:*',
-      'reject 169.254.0.0/16:*',
-      'reject 127.0.0.0/8:*',
-      'reject 192.168.0.0/16:*',
-      'reject 10.0.0.0/8:*',
-      'reject 172.16.0.0/12:*',
-      # this is where 'reject [public_addr]:*' may or may not be
-      'reject *:25',
-      'reject *:119',
-      'reject *:135-139',
-      'reject *:445',
-      'reject *:563',
-      'reject *:1214',
-      'reject *:4661-4666',
-      'reject *:6346-6429',
-      'reject *:6699',
-      'reject *:6881-6999',
-      'accept *:*',
-    )
-
-    runner = test.runner.get_runner()
-
-    with runner.get_tor_controller() as controller:
-      # We can't simply compare the policies because the tor policy may or may
-      # not have a reject entry for our public address. Hence, stripping it
-      # from the policy's string, then comparing those.
-
-      policy_str = str(controller.get_exit_policy())
-
-      public_addr_start = policy_str.find('reject 172.16.0.0/12:*') + 22
-      public_addr_end = policy_str.find(', reject *:25')
-
-      policy_str = policy_str[:public_addr_start] + policy_str[public_addr_end:]
-      self.assertEqual(str(expected), policy_str)
+    with test.runner.get_runner().get_tor_controller() as controller:
+      self.assertEqual(ExitPolicy('reject *:*'), controller.get_exit_policy())
 
   @test.require.controller
   def test_authenticate(self):
@@ -364,10 +331,10 @@ class TestController(unittest.TestCase):
       control_socket = controller.get_socket()
 
       if isinstance(control_socket, stem.socket.ControlPort):
-        connection_value = str(control_socket.get_port())
+        connection_value = str(control_socket.port)
         config_key = 'ControlPort'
       elif isinstance(control_socket, stem.socket.ControlSocketFile):
-        connection_value = str(control_socket.get_socket_path())
+        connection_value = control_socket.path
         config_key = 'ControlSocket'
 
       # successful single query
@@ -479,6 +446,7 @@ class TestController(unittest.TestCase):
             'HiddenServiceVersion': '2',
           },
           service2_path: {
+            'HiddenServiceVersion': '2',
             'HiddenServiceAuthorizeClient': 'stealth a, b',
             'HiddenServicePort': [
               (8030, '127.0.0.1', 8030),
@@ -563,24 +531,26 @@ class TestController(unittest.TestCase):
 
   @test.require.controller
   @test.require.version(Requirement.ADD_ONION)
-  def test_with_ephemeral_hidden_services(self):
+  def test_with_invalid_ephemeral_hidden_service_port(self):
+    with test.runner.get_runner().get_tor_controller() as controller:
+      for ports in (4567890, [4567, 4567890], {4567: '-:4567'}):
+        exc_msg = "ADD_ONION response didn't have an OK status: Invalid VIRTPORT/TARGET"
+        self.assertRaisesWith(stem.ProtocolError, exc_msg, controller.create_ephemeral_hidden_service, ports)
+
+  @test.require.controller
+  @test.require.version(Requirement.ADD_ONION)
+  def test_ephemeral_hidden_services_v2(self):
     """
-    Exercises creating ephemeral hidden services and methods when they're
-    present.
+    Exercises creating v2 ephemeral hidden services.
     """
 
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
-      # try creating a service with an invalid ports
-
-      for ports in (4567890, [4567, 4567890], {4567: '-:4567'}):
-        exc_msg = "ADD_ONION response didn't have an OK status: Invalid VIRTPORT/TARGET"
-        self.assertRaisesRegexp(stem.ProtocolError, exc_msg, controller.create_ephemeral_hidden_service, ports)
-
-      response = controller.create_ephemeral_hidden_service(4567)
+      response = controller.create_ephemeral_hidden_service(4567, key_content = 'RSA1024')
       self.assertEqual([response.service_id], controller.list_ephemeral_hidden_services())
       self.assertTrue(response.private_key is not None)
+      self.assertEqual('RSA1024', response.private_key_type)
       self.assertEqual({}, response.client_auth)
 
       # drop the service
@@ -601,7 +571,52 @@ class TestController(unittest.TestCase):
 
       # create a service where we never see the private key
 
-      response = controller.create_ephemeral_hidden_service(4568, discard_key = True)
+      response = controller.create_ephemeral_hidden_service(4568, key_content = 'RSA1024', discard_key = True)
+      self.assertTrue(response.service_id in controller.list_ephemeral_hidden_services())
+      self.assertEqual(None, response.private_key)
+      self.assertEqual(None, response.private_key_type)
+
+      # other controllers shouldn't be able to see these hidden services
+
+      with runner.get_tor_controller() as second_controller:
+        self.assertEqual(2, len(controller.list_ephemeral_hidden_services()))
+        self.assertEqual(0, len(second_controller.list_ephemeral_hidden_services()))
+
+  @test.require.controller
+  @test.require.version(Requirement.HIDDEN_SERVICE_V3)
+  def test_ephemeral_hidden_services_v3(self):
+    """
+    Exercises creating v3 ephemeral hidden services.
+    """
+
+    runner = test.runner.get_runner()
+
+    with runner.get_tor_controller() as controller:
+      response = controller.create_ephemeral_hidden_service(4567, key_content = 'ED25519-V3')
+      self.assertEqual([response.service_id], controller.list_ephemeral_hidden_services())
+      self.assertTrue(response.private_key is not None)
+      self.assertEqual('ED25519-V3', response.private_key_type)
+      self.assertEqual({}, response.client_auth)
+
+      # drop the service
+
+      self.assertEqual(True, controller.remove_ephemeral_hidden_service(response.service_id))
+      self.assertEqual([], controller.list_ephemeral_hidden_services())
+
+      # recreate the service with the same private key
+
+      recreate_response = controller.create_ephemeral_hidden_service(4567, key_type = response.private_key_type, key_content = response.private_key)
+      self.assertEqual([response.service_id], controller.list_ephemeral_hidden_services())
+      self.assertEqual(response.service_id, recreate_response.service_id)
+
+      # the response only includes the private key when making a new one
+
+      self.assertEqual(None, recreate_response.private_key)
+      self.assertEqual(None, recreate_response.private_key_type)
+
+      # create a service where we never see the private key
+
+      response = controller.create_ephemeral_hidden_service(4568, key_content = 'ED25519-V3', discard_key = True)
       self.assertTrue(response.service_id in controller.list_ephemeral_hidden_services())
       self.assertEqual(None, response.private_key)
       self.assertEqual(None, response.private_key_type)
@@ -644,7 +659,7 @@ class TestController(unittest.TestCase):
 
     with runner.get_tor_controller() as controller:
       exc_msg = "ADD_ONION response didn't have an OK status: No auth clients specified"
-      self.assertRaisesRegexp(stem.ProtocolError, exc_msg, controller.create_ephemeral_hidden_service, 4567, basic_auth = {})
+      self.assertRaisesWith(stem.ProtocolError, exc_msg, controller.create_ephemeral_hidden_service, 4567, basic_auth = {})
 
   @test.require.controller
   @test.require.version(Requirement.ADD_ONION)
@@ -775,8 +790,8 @@ class TestController(unittest.TestCase):
     """
 
     with test.runner.get_runner().get_tor_controller() as controller:
-      self.assertRaisesRegexp(stem.InvalidArguments, "DisableAllSwap cannot be changed while tor's running", controller.set_conf, 'DisableAllSwap', '1')
-      self.assertRaisesRegexp(stem.InvalidArguments, "DisableAllSwap, User cannot be changed while tor's running", controller.set_options, {'User': 'atagar', 'DisableAllSwap': '1'})
+      self.assertRaisesWith(stem.InvalidArguments, "DisableAllSwap cannot be changed while tor's running", controller.set_conf, 'DisableAllSwap', '1')
+      self.assertRaisesWith(stem.InvalidArguments, "DisableAllSwap, User cannot be changed while tor's running", controller.set_options, {'User': 'atagar', 'DisableAllSwap': '1'})
 
   @test.require.controller
   @test.require.version(Requirement.LOADCONF)
@@ -845,7 +860,7 @@ class TestController(unittest.TestCase):
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
-      self.assertEqual([], controller.get_ports(Listener.OR))
+      self.assertEqual([test.runner.ORPORT], controller.get_ports(Listener.OR))
       self.assertEqual([], controller.get_ports(Listener.DIR))
       self.assertEqual([test.runner.SOCKS_PORT], controller.get_ports(Listener.SOCKS))
       self.assertEqual([], controller.get_ports(Listener.TRANS))
@@ -866,7 +881,7 @@ class TestController(unittest.TestCase):
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
-      self.assertEqual([], controller.get_listeners(Listener.OR))
+      self.assertEqual([('0.0.0.0', test.runner.ORPORT)], controller.get_listeners(Listener.OR))
       self.assertEqual([], controller.get_listeners(Listener.DIR))
       self.assertEqual([('127.0.0.1', test.runner.SOCKS_PORT)], controller.get_listeners(Listener.SOCKS))
       self.assertEqual([], controller.get_listeners(Listener.TRANS))
@@ -1071,6 +1086,7 @@ class TestController(unittest.TestCase):
   @test.require.controller
   @test.require.online
   def test_mapaddress(self):
+    self.skipTest('(https://trac.torproject.org/projects/tor/ticket/25611)')
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
@@ -1087,7 +1103,7 @@ class TestController(unittest.TestCase):
           s.settimeout(30)
           s.connect(('127.0.0.1', int(controller.get_conf('SocksPort'))))
           test.network.negotiate_socks(s, '1.2.1.2', 80)
-          s.sendall(stem.util.str_tools._to_bytes(test.network.ip_request))  # make the http request for the ip address
+          s.sendall(stem.util.str_tools._to_bytes(test.network.IP_REQUEST))  # make the http request for the ip address
           response = s.recv(1000)
 
           if response:
@@ -1132,6 +1148,7 @@ class TestController(unittest.TestCase):
       self.assertEqual(md_by_fingerprint, md_by_nickname)
 
   @test.require.controller
+  @test.require.online
   def test_get_microdescriptors(self):
     """
     Fetches a few descriptors via the get_microdescriptors() method.
@@ -1139,7 +1156,7 @@ class TestController(unittest.TestCase):
 
     runner = test.runner.get_runner()
 
-    if not os.path.exists(runner.get_test_dir('cached-descriptors')):
+    if not os.path.exists(runner.get_test_dir('cached-microdescs')):
       self.skipTest('(no cached microdescriptors)')
       return
 
@@ -1278,7 +1295,7 @@ class TestController(unittest.TestCase):
       # try to fetch something that doesn't exist
 
       exc_msg = 'No running hidden service at m4cfuk6qp4lpu2g3.onion'
-      self.assertRaisesRegexp(stem.DescriptorUnavailable, exc_msg, controller.get_hidden_service_descriptor, 'm4cfuk6qp4lpu2g3')
+      self.assertRaisesWith(stem.DescriptorUnavailable, exc_msg, controller.get_hidden_service_descriptor, 'm4cfuk6qp4lpu2g3')
 
       # ... but shouldn't fail if we have a default argument or aren't awaiting the descriptor
 
@@ -1348,16 +1365,20 @@ class TestController(unittest.TestCase):
     """
 
     with test.runner.get_runner().get_tor_controller() as controller:
-      self.assertEqual(None, controller.get_conf('OrPort'))
+      try:
+        controller.reset_conf('OrPort', 'DisableNetwork')
+        self.assertEqual(None, controller.get_conf('OrPort'))
 
-      # DisableNetwork ensures no port is actually opened
-      controller.set_options({'OrPort': '9090', 'DisableNetwork': '1'})
+        # DisableNetwork ensures no port is actually opened
+        controller.set_options({'OrPort': '9090', 'DisableNetwork': '1'})
 
-      # TODO once tor 0.2.7.x exists, test that we can generate a descriptor on demand.
+        # TODO once tor 0.2.7.x exists, test that we can generate a descriptor on demand.
 
-      self.assertEqual('9090', controller.get_conf('OrPort'))
-      controller.reset_conf('OrPort', 'DisableNetwork')
-      self.assertEqual(None, controller.get_conf('OrPort'))
+        self.assertEqual('9090', controller.get_conf('OrPort'))
+        controller.reset_conf('OrPort', 'DisableNetwork')
+        self.assertEqual(None, controller.get_conf('OrPort'))
+      finally:
+        controller.set_conf('OrPort', str(test.runner.ORPORT))
 
   def _get_router_status_entry(self, controller):
     """
