@@ -8,6 +8,7 @@ import os
 import tempfile
 import unittest
 
+import stem.prereq
 import stem.util.proc
 import stem.util.system
 import test.require
@@ -101,7 +102,7 @@ class TestSystem(unittest.TestCase):
     self.assertEqual(State.PENDING, task.status)
 
     task.run()
-    self.assertRaisesRegexp(RuntimeError, 'hello world', task.join)
+    self.assertRaisesWith(RuntimeError, 'hello world', task.join)
     self.assertEqual(State.FAILED, task.status)
     self.assertTrue(0 < task.runtime < 1.0)
 
@@ -530,7 +531,7 @@ class TestSystem(unittest.TestCase):
     self.assertEqual(os.path.join(home_dir, 'foo'), stem.util.system.expand_path('~%s/foo' % username))
 
   def test_call_timeout(self):
-    self.assertRaisesRegexp(stem.util.system.CallTimeoutError, "Process didn't finish after 0.0 seconds", stem.util.system.call, 'sleep 1', timeout = 0.001)
+    self.assertRaisesWith(stem.util.system.CallTimeoutError, "Process didn't finish after 0.0 seconds", stem.util.system.call, 'sleep 1', timeout = 0.001)
 
   def test_call_time_tracked(self):
     """
@@ -546,6 +547,10 @@ class TestSystem(unittest.TestCase):
     Exercises the get_process_name() and set_process_name() methods.
     """
 
+    if stem.prereq.is_pypy():
+      self.skipTest('(unimplemented for pypy)')
+      return
+
     initial_name = stem.util.system.get_process_name()
     self.assertTrue('run_tests.py' in initial_name)
 
