@@ -1,4 +1,4 @@
-# Copyright 2011-2017, Damian Johnson and The Tor Project
+# Copyright 2011-2018, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -58,14 +58,19 @@ CONFIG = stem.util.conf.config_dict('test', {
 })
 
 SOCKS_PORT = 1112
+ORPORT = 1113
 
 BASE_TORRC = """# configuration for stem integration tests
 DataDirectory %%s
 SocksPort %i
+ORPort %i
+ExitRelay 0
+PublishServerDescriptor 0
+AssumeReachable 1
 DownloadExtraInfo 1
 Log notice stdout
 Log notice file %%s/tor_log
-""" % SOCKS_PORT
+""" % (SOCKS_PORT, ORPORT)
 
 # singleton Runner instance
 INTEG_RUNNER = None
@@ -156,7 +161,7 @@ class Runner(object):
     self._tor_cmd = None
     self._tor_cwd = ''
     self._torrc_contents = ''
-    self._custom_opts = None
+    self._custom_opts = []
     self._tor_process = None
     self._chroot_path = None
 
@@ -282,6 +287,9 @@ class Runner(object):
         except OSError:
           pass
 
+        self._tor_process.stdout.close()
+        self._tor_process.stderr.close()
+
         self._tor_process.wait()  # blocks until the process is done
 
       # if we've made a temporary data directory then clean it up
@@ -303,7 +311,7 @@ class Runner(object):
       self._tor_cmd = None
       self._tor_cwd = ''
       self._torrc_contents = ''
-      self._custom_opts = None
+      self._custom_opts = []
       self._tor_process = None
 
       println('done', STATUS)
@@ -485,6 +493,21 @@ class Runner(object):
 
     return os.path.basename(self._get('_tor_cmd')) if base_cmd else self._get('_tor_cmd')
 
+  def assert_tor_is_running(self):
+    """
+    Checks if our tor process is running. If not, this prints an error and
+    provides **False**.
+    """
+
+    process_status = self._tor_process.poll()  # None if running
+
+    if process_status is None:
+      return True
+    else:
+      process_output = (self._tor_process.stdout.read() + '\n\n' + self._tor_process.stderr.read()).strip()
+      println('\n%s\nOur tor process ended prematurely with exit status %s\n%s\n\n%s' % ('=' * 60, process_status, '=' * 60, process_output), ERROR)
+      return False
+
   def _get(self, attr):
     """
     Fetches one of our attributes in a thread safe manner, raising if we aren't
@@ -603,9 +626,10 @@ class Runner(object):
       self._tor_process = stem.process.launch_tor(
         tor_cmd = tor_cmd,
         torrc_path = os.path.join(self._test_dir, 'torrc'),
-        completion_percent = 100 if test.Target.ONLINE in self.attribute_targets else 5,
+        completion_percent = 100 if test.Target.ONLINE in self.attribute_targets else 0,
         init_msg_handler = lambda line: println('  %s' % line, SUBSTATUS),
         take_ownership = True,
+        close_output = False,
       )
 
       runtime = time.time() - start_time
