@@ -5,17 +5,17 @@ integ tests, but a few bits lend themselves to unit testing.
 
 import datetime
 import io
-import time
 import unittest
 
 import stem.descriptor.router_status_entry
 import stem.response
+import stem.response.events
 import stem.socket
 import stem.util.system
 import stem.version
 
 from stem import ControllerError, DescriptorUnavailable, InvalidArguments, InvalidRequest, ProtocolError, UnsatisfiableRequest
-from stem.control import _parse_circ_path, Listener, Controller, EventType
+from stem.control import MALFORMED_EVENTS, _parse_circ_path, Listener, Controller, EventType
 from stem.response import ControlMessage
 from stem.exit_policy import ExitPolicy
 
@@ -26,15 +26,32 @@ except ImportError:
   from mock import Mock, patch
 
 NS_DESC = 'r %s %s u5lTXJKGsLKufRLnSyVqT7TdGYw 2012-12-30 22:02:49 77.223.43.54 9001 0\ns Fast Named Running Stable Valid\nw Bandwidth=75'
+TEST_TIMESTAMP = 12345
+
+BW_EVENT = ControlMessage.from_str('650 BW 15 25', 'EVENT', normalize = True, arrived_at = TEST_TIMESTAMP)
+CIRC_EVENT = ControlMessage.from_str('650 CIRC 4 LAUNCHED', 'EVENT', normalize = True, arrived_at = TEST_TIMESTAMP)
+BAD_EVENT = ControlMessage.from_str('650 BW &15* 25', normalize = True, arrived_at = TEST_TIMESTAMP)
 
 
 class TestControl(unittest.TestCase):
   def setUp(self):
     socket = stem.socket.ControlSocket()
 
-    with patch('stem.control.Controller.add_event_listener', Mock()):
+    # When initially constructing a controller we need to suppress msg, so our
+    # constructor's SETEVENTS requests pass.
+
+    with patch('stem.control.BaseController.msg', Mock()):
       self.controller = Controller(socket)
 
+      self.circ_listener = Mock()
+      self.controller.add_event_listener(self.circ_listener, EventType.CIRC)
+
+      self.bw_listener = Mock()
+      self.controller.add_event_listener(self.bw_listener, EventType.BW)
+
+      self.malformed_listener = Mock()
+      self.controller.add_event_listener(self.malformed_listener, MALFORMED_EVENTS)
+
   def test_event_description(self):
     self.assertEqual("Logging at the debug runlevel. This is low level, high volume information about tor's internals that generally isn't useful to users.", stem.control.event_description('DEBUG'))
     self.assertEqual('Event emitted every second with the bytes sent and received by tor.', stem.control.event_description('BW'))
@@ -53,28 +70,32 @@ class TestControl(unittest.TestCase):
 
   @patch('stem.control.Controller.msg')
   def test_get_info_address_caching(self, msg_mock):
-    test_start = time.time()
     msg_mock.return_value = ControlMessage.from_str('551 Address unknown\r\n')
 
-    self.assertEqual(0, self.controller._address_cached_at)
     self.assertEqual(None, self.controller._last_address_exc)
-    self.assertRaisesRegexp(stem.ProtocolError, 'Address unknown', self.controller.get_info, 'address')
-    self.assertTrue(test_start <= self.controller._address_cached_at <= time.time())
-    self.assertEqual("GETINFO response didn't have an OK status:\nAddress unknown", str(self.controller._last_address_exc))
+    self.assertRaisesWith(stem.OperationFailed, 'Address unknown', self.controller.get_info, 'address')
+    self.assertEqual('Address unknown', str(self.controller._last_address_exc))
     self.assertEqual(1, msg_mock.call_count)
 
     # now that we have a cached failure we should provide that back
 
-    self.assertRaisesRegexp(stem.ProtocolError, 'Address unknown', self.controller.get_info, 'address')
+    self.assertRaisesWith(stem.OperationFailed, 'Address unknown', self.controller.get_info, 'address')
     self.assertEqual(1, msg_mock.call_count)
 
-    # pretend it's a minute later and try again, now with an address
+    # invalidates the cache, transitioning from no address to having one
 
     msg_mock.return_value = ControlMessage.from_str('250-address=17.2.89.80\r\n250 OK\r\n', 'GETINFO')
-    self.assertRaisesRegexp(stem.ProtocolError, 'Address unknown', self.controller.get_info, 'address')
-    self.controller._address_cached_at -= 60
+    self.assertRaisesWith(stem.OperationFailed, 'Address unknown', self.controller.get_info, 'address')
+    self.controller._handle_event(ControlMessage.from_str('650 STATUS_SERVER NOTICE EXTERNAL_ADDRESS ADDRESS=17.2.89.80 METHOD=DIRSERV\r\n'))
     self.assertEqual('17.2.89.80', self.controller.get_info('address'))
 
+    # invalidates the cache, transitioning from one address to another
+
+    msg_mock.return_value = ControlMessage.from_str('250-address=80.89.2.17\r\n250 OK\r\n', 'GETINFO')
+    self.assertEqual('17.2.89.80', self.controller.get_info('address'))
+    self.controller._handle_event(ControlMessage.from_str('650 STATUS_SERVER NOTICE EXTERNAL_ADDRESS ADDRESS=80.89.2.17 METHOD=DIRSERV\r\n'))
+    self.assertEqual('80.89.2.17', self.controller.get_info('address'))
+
   @patch('stem.control.Controller.msg')
   @patch('stem.control.Controller.get_conf')
   def test_get_info_without_fingerprint(self, get_conf_mock, msg_mock):
@@ -82,19 +103,19 @@ class TestControl(unittest.TestCase):
     get_conf_mock.return_value = None
 
     self.assertEqual(None, self.controller._last_fingerprint_exc)
-    self.assertRaisesRegexp(stem.ProtocolError, 'Not running in server mode', self.controller.get_info, 'fingerprint')
-    self.assertEqual("GETINFO response didn't have an OK status:\nNot running in server mode", str(self.controller._last_fingerprint_exc))
+    self.assertRaisesWith(stem.OperationFailed, 'Not running in server mode', self.controller.get_info, 'fingerprint')
+    self.assertEqual('Not running in server mode', str(self.controller._last_fingerprint_exc))
     self.assertEqual(1, msg_mock.call_count)
 
     # now that we have a cached failure we should provide that back
 
-    self.assertRaisesRegexp(stem.ProtocolError, 'Not running in server mode', self.controller.get_info, 'fingerprint')
+    self.assertRaisesWith(stem.OperationFailed, 'Not running in server mode', self.controller.get_info, 'fingerprint')
     self.assertEqual(1, msg_mock.call_count)
 
     # ... but if we become a relay we'll call it again
 
     get_conf_mock.return_value = '443'
-    self.assertRaisesRegexp(stem.ProtocolError, 'Not running in server mode', self.controller.get_info, 'fingerprint')
+    self.assertRaisesWith(stem.OperationFailed, 'Not running in server mode', self.controller.get_info, 'fingerprint')
     self.assertEqual(2, msg_mock.call_count)
 
   @patch('stem.control.Controller.get_info')
@@ -152,35 +173,17 @@ class TestControl(unittest.TestCase):
       self.controller._is_caching_enabled = True
 
   @patch('stem.control.Controller.get_info')
-  @patch('stem.control.Controller.get_conf')
-  def test_get_exit_policy(self, get_conf_mock, get_info_mock):
+  def test_get_exit_policy(self, get_info_mock):
     """
     Exercises the get_exit_policy() method.
     """
 
-    get_conf_mock.side_effect = lambda param, **kwargs: {
-      'ExitPolicyRejectPrivate': '1',
-      'ExitPolicy': ['accept *:80,   accept *:443', 'accept 43.5.5.5,reject *:22'],
-    }[param]
-
     get_info_mock.side_effect = lambda param, default = None: {
-      'address': '123.45.67.89',
-      'exit-policy/default': 'reject *:25,reject *:119,reject *:135-139,reject *:445,reject *:563,reject *:1214,reject *:4661-4666,reject *:6346-6429,reject *:6699,reject *:6881-6999,accept *:*',
+      'exit-policy/full': 'reject *:25,reject *:119,reject *:135-139,reject *:445,reject *:563,reject *:1214,reject *:4661-4666,reject *:6346-6429,reject *:6699,reject *:6881-6999,accept *:*',
     }[param]
 
     expected = ExitPolicy(
-      'reject 0.0.0.0/8:*',  # private entries
-      'reject 169.254.0.0/16:*',
-      'reject 127.0.0.0/8:*',
-      'reject 192.168.0.0/16:*',
-      'reject 10.0.0.0/8:*',
-      'reject 172.16.0.0/12:*',
-      'reject 123.45.67.89:*',  # relay's public address
-      'accept *:80',  # finally we get to our ExitPolicy
-      'accept *:443',
-      'accept 43.5.5.5:*',
-      'reject *:22',
-      'reject *:25',  # default policy
+      'reject *:25',
       'reject *:119',
       'reject *:135-139',
       'reject *:445',
@@ -193,7 +196,58 @@ class TestControl(unittest.TestCase):
       'accept *:*',
     )
 
-    self.assertEqual(expected, self.controller.get_exit_policy())
+    self.assertEqual(str(expected), str(self.controller.get_exit_policy()))
+
+  @patch('stem.control.Controller.get_info')
+  @patch('stem.control.Controller.get_conf')
+  def test_get_exit_policy_if_not_relaying(self, get_conf_mock, get_info_mock):
+    # If tor lacks an ORPort, resolved extrnal address, hasn't finished making
+    # our server descriptor (ie. tor just started), etc 'GETINFO
+    # exit-policy/full' will fail.
+
+    get_conf_mock.side_effect = lambda param, **kwargs: {
+      'ExitRelay': '1',
+      'ExitPolicyRejectPrivate': '1',
+      'ExitPolicy': ['accept *:80,   accept *:443', 'accept 43.5.5.5,reject *:22'],
+    }[param]
+
+    expected = ExitPolicy(
+      'reject 0.0.0.0/8:*',
+      'reject 169.254.0.0/16:*',
+      'reject 127.0.0.0/8:*',
+      'reject 192.168.0.0/16:*',
+      'reject 10.0.0.0/8:*',
+      'reject 172.16.0.0/12:*',
+      'reject 1.2.3.4:*',
+      'accept *:80',
+      'accept *:443',
+      'accept 43.5.5.5:*',
+      'reject *:22',
+    )
+
+    # Unfortunate it's a bit tricky to have a mock that raises exceptions in
+    # response to some arguments, and returns a response for others. As such
+    # mapping it to the following function.
+
+    exit_policy_exception = None
+
+    def getinfo_response(param, default = None):
+      if param == 'address':
+        return '1.2.3.4'
+      elif param == 'exit-policy/default':
+        return ''
+      elif param == 'exit-policy/full' and exit_policy_exception:
+        raise exit_policy_exception
+      else:
+        raise ValueError("Unmocked request for 'GETINFO %s'" % param)
+
+    get_info_mock.side_effect = getinfo_response
+
+    exit_policy_exception = stem.OperationFailed('552', 'Not running in server mode')
+    self.assertEqual(str(expected), str(self.controller.get_exit_policy()))
+
+    exit_policy_exception = stem.OperationFailed('551', 'Descriptor still rebuilding - not ready yet')
+    self.assertEqual(str(expected), str(self.controller.get_exit_policy()))
 
   @patch('stem.control.Controller.get_info')
   @patch('stem.control.Controller.get_conf')
@@ -519,7 +573,7 @@ class TestControl(unittest.TestCase):
     get_info_mock.side_effect = ControllerError('nope, too bad')
 
     exc_msg = 'Unable to determine our own fingerprint: nope, too bad'
-    self.assertRaisesRegexp(ControllerError, exc_msg, self.controller.get_network_status)
+    self.assertRaisesWith(ControllerError, exc_msg, self.controller.get_network_status)
     self.assertEqual('boom', self.controller.get_network_status(default = 'boom'))
 
     # successful request
@@ -542,7 +596,7 @@ class TestControl(unittest.TestCase):
     get_info_mock.side_effect = InvalidArguments(None, 'GETINFO request contained unrecognized keywords: ns/id/5AC9C5AA75BA1F18D8459B326B4B8111A856D290')
 
     exc_msg = "Tor was unable to provide the descriptor for '5AC9C5AA75BA1F18D8459B326B4B8111A856D290'"
-    self.assertRaisesRegexp(DescriptorUnavailable, exc_msg, self.controller.get_network_status, '5AC9C5AA75BA1F18D8459B326B4B8111A856D290')
+    self.assertRaisesWith(DescriptorUnavailable, exc_msg, self.controller.get_network_status, '5AC9C5AA75BA1F18D8459B326B4B8111A856D290')
 
   @patch('stem.control.Controller.get_info')
   def test_get_network_status(self, get_info_mock):
@@ -593,7 +647,7 @@ class TestControl(unittest.TestCase):
   @patch('stem.control.Controller.is_authenticated', Mock(return_value = True))
   @patch('stem.control.Controller._attach_listeners', Mock(return_value = ([], [])))
   @patch('stem.control.Controller.get_version')
-  def test_event_listening(self, get_version_mock):
+  def test_add_event_listener(self, get_version_mock):
     """
     Exercises the add_event_listener and remove_event_listener methods.
     """
@@ -612,8 +666,74 @@ class TestControl(unittest.TestCase):
     self.controller.add_event_listener(Mock(), EventType.BW)
 
     # EventType.SIGNAL was added in tor version 0.2.3.1-alpha
+
     self.assertRaises(InvalidRequest, self.controller.add_event_listener, Mock(), EventType.SIGNAL)
 
+  @patch('time.time', Mock(return_value = TEST_TIMESTAMP))
+  def test_events_get_received(self):
+    """
+    Trigger an event, checking that our listeners get notified.
+    """
+
+    self._emit_event(CIRC_EVENT)
+    self.circ_listener.assert_called_once_with(CIRC_EVENT)
+    self.bw_listener.assert_not_called()
+    self.malformed_listener.assert_not_called()
+
+    self._emit_event(BW_EVENT)
+    self.bw_listener.assert_called_once_with(BW_EVENT)
+
+  @patch('time.time', Mock(return_value = TEST_TIMESTAMP))
+  def test_event_listing_with_error(self):
+    """
+    Raise an exception in an event listener to confirm it doesn't break our
+    event thread.
+    """
+
+    self.circ_listener.side_effect = ValueError('boom')
+
+    self._emit_event(CIRC_EVENT)
+    self.circ_listener.assert_called_once_with(CIRC_EVENT)
+    self.bw_listener.assert_not_called()
+    self.malformed_listener.assert_not_called()
+
+    self._emit_event(BW_EVENT)
+    self.bw_listener.assert_called_once_with(BW_EVENT)
+
+  @patch('time.time', Mock(return_value = TEST_TIMESTAMP))
+  def test_event_listing_with_malformed_event(self):
+    """
+    Attempt to parse a malformed event emitted from Tor. It's important this
+    doesn't break our event thread.
+    """
+
+    # When stem.response.convert() encounters malformed content we still recast
+    # the message.
+
+    expected_bad_event = ControlMessage.from_str(BAD_EVENT.raw_content())
+    setattr(expected_bad_event, 'arrived_at', TEST_TIMESTAMP)
+    expected_bad_event.__class__ = stem.response.events.BandwidthEvent
+
+    self._emit_event(BAD_EVENT)
+    self.circ_listener.assert_not_called()
+    self.bw_listener.assert_not_called()
+    self.malformed_listener.assert_called_once_with(expected_bad_event)
+
+    self._emit_event(BW_EVENT)
+    self.bw_listener.assert_called_once_with(BW_EVENT)
+
+  @patch('stem.control.Controller.get_version', Mock(return_value = stem.version.Version('0.5.0.14')))
+  @patch('stem.control.Controller.msg', Mock(return_value = ControlMessage.from_str('250 OK\r\n')))
+  @patch('stem.control.Controller.add_event_listener', Mock())
+  @patch('stem.control.Controller.remove_event_listener', Mock())
+  def test_timeout(self):
+    """
+    Methods that have an 'await' argument also have an optional timeout. Check
+    that we raise a Timeout exception when it's elapsed.
+    """
+
+    self.assertRaisesWith(stem.Timeout, 'Reached our 0.1 second timeout', self.controller.get_hidden_service_descriptor, '5g2upl4pq6kufc4m', await_result = True, timeout = 0.1)
+
   def test_get_streams(self):
     """
     Exercises the get_streams() method.
@@ -726,3 +846,28 @@ class TestControl(unittest.TestCase):
     with patch('stem.control.Controller.msg', Mock(return_value = None)):
       get_version_mock.return_value = stem.version.Version('0.2.5.2')
       self.controller.drop_guards()
+
+  def _emit_event(self, event):
+    # Spins up our Controller's thread pool, emits an event, then shuts it
+    # down. This last part is important for a couple reasons...
+    #
+    #   1. So we don't leave any lingering threads.
+    #
+    #   2. To ensure our event handlers are done being executed. Events are
+    #      processed asynchronously, so the only way to endsure it's done
+    #      with its work is to join on the thread.
+
+    with patch('stem.control.Controller.is_alive') as is_alive_mock:
+      is_alive_mock.return_value = True
+      self.controller._launch_threads()
+
+      try:
+        # Converting an event back into an uncast ControlMessage, then feeding it
+        # into our controller's event queue.
+
+        uncast_event = ControlMessage.from_str(event.raw_content())
+        self.controller._event_queue.put(uncast_event)
+        self.controller._event_notice.set()
+      finally:
+        is_alive_mock.return_value = False
+        self.controller._close()
