@@ -4,7 +4,6 @@ Unit tests for the NetworkStatusDocumentV3 of stem.descriptor.networkstatus.
 
 import datetime
 import io
-import re
 import unittest
 
 import stem.descriptor
@@ -12,7 +11,6 @@ import stem.version
 import test.require
 
 from stem import Flag
-from stem.util import str_type
 
 from stem.descriptor import CRYPTO_BLOB
 
@@ -368,7 +366,7 @@ DnN5aFtYKiTc19qIC7Nmo+afPdDEf0MlJvEOP5EWl3w=
 
   @test.require.cryptography
   def test_descriptor_signing(self):
-    self.assertRaisesRegexp(NotImplementedError, 'Signing of NetworkStatusDocumentV3 not implemented', NetworkStatusDocumentV3.create, sign = True)
+    self.assertRaisesWith(NotImplementedError, 'Signing of NetworkStatusDocumentV3 not implemented', NetworkStatusDocumentV3.create, sign = True)
 
   def test_examples(self):
     """
@@ -414,7 +412,7 @@ DnN5aFtYKiTc19qIC7Nmo+afPdDEf0MlJvEOP5EWl3w=
     # change a relay's nickname in the consensus so it's no longer validly signed
 
     consensus = stem.descriptor.networkstatus.NetworkStatusDocumentV3(consensus_content.replace(b'test002r', b'different_nickname'))
-    self.assertRaisesRegexp(ValueError, 'Network Status Document has 0 valid signatures out of 2 total, needed 1', consensus.validate_signatures, certs)
+    self.assertRaisesWith(ValueError, 'Network Status Document has 0 valid signatures out of 2 total, needed 1', consensus.validate_signatures, certs)
 
   def test_handlers(self):
     """
@@ -783,19 +781,19 @@ DnN5aFtYKiTc19qIC7Nmo+afPdDEf0MlJvEOP5EWl3w=
 
     test_values = (
       ('', {}),
-      ('fast-speed=40960', {str_type('fast-speed'): 40960}),    # numeric value
-      ('guard-wfu=94.669%', {str_type('guard-wfu'): 0.94669}),  # percentage value
-      ('guard-wfu=94.669% guard-tk=691200', {str_type('guard-wfu'): 0.94669, str_type('guard-tk'): 691200}),  # multiple values
+      ('fast-speed=40960', {'fast-speed': 40960}),    # numeric value
+      ('guard-wfu=94.669%', {'guard-wfu': 0.94669}),  # percentage value
+      ('guard-wfu=94.669% guard-tk=691200', {'guard-wfu': 0.94669, 'guard-tk': 691200}),  # multiple values
       ('stable-uptime=0 stable-mtbf=0 fast-speed=0 guard-wfu=0.000% guard-tk=0 guard-bw-inc-exits=0 guard-bw-exc-exits=0 enough-mtbf=1 ignoring-advertised-bws=0', {
-        str_type('stable-uptime'): 0,
-        str_type('stable-mtbf'): 0,
-        str_type('fast-speed'): 0,
-        str_type('guard-wfu'): 0.0,
-        str_type('guard-tk'): 0,
-        str_type('guard-bw-inc-exits'): 0,
-        str_type('guard-bw-exc-exits'): 0,
-        str_type('enough-mtbf'): 1,
-        str_type('ignoring-advertised-bws'): 0,
+        'stable-uptime': 0,
+        'stable-mtbf': 0,
+        'fast-speed': 0,
+        'guard-wfu': 0.0,
+        'guard-tk': 0,
+        'guard-bw-inc-exits': 0,
+        'guard-bw-exc-exits': 0,
+        'enough-mtbf': 1,
+        'ignoring-advertised-bws': 0,
       }),
     )
 
@@ -808,14 +806,14 @@ DnN5aFtYKiTc19qIC7Nmo+afPdDEf0MlJvEOP5EWl3w=
     full_line = 'stable-uptime=693369 stable-mtbf=153249 fast-speed=40960 guard-wfu=94.669% guard-tk=691200 guard-bw-inc-exits=174080 guard-bw-exc-exits=184320 enough-mtbf=1'
 
     expected_value = {
-      str_type('stable-uptime'): 693369,
-      str_type('stable-mtbf'): 153249,
-      str_type('fast-speed'): 40960,
-      str_type('guard-wfu'): 0.94669,
-      str_type('guard-tk'): 691200,
-      str_type('guard-bw-inc-exits'): 174080,
-      str_type('guard-bw-exc-exits'): 184320,
-      str_type('enough-mtbf'): 1,
+      'stable-uptime': 693369,
+      'stable-mtbf': 153249,
+      'fast-speed': 40960,
+      'guard-wfu': 0.94669,
+      'guard-tk': 691200,
+      'guard-bw-inc-exits': 174080,
+      'guard-bw-exc-exits': 184320,
+      'enough-mtbf': 1,
     }
 
     document = NetworkStatusDocumentV3.create({'vote-status': 'vote', 'flag-thresholds': full_line})
@@ -1247,7 +1245,7 @@ DnN5aFtYKiTc19qIC7Nmo+afPdDEf0MlJvEOP5EWl3w=
 
     for attr, expected_exception in test_values:
       content = DirectoryAuthority.content(attr)
-      self.assertRaisesRegexp(ValueError, re.escape(expected_exception), DirectoryAuthority, content, True)
+      self.assertRaisesWith(ValueError, expected_exception, DirectoryAuthority, content, True)
 
       authority = DirectoryAuthority(content, False)
       self.assertEqual([], authority.shared_randomness_commitments)
@@ -1256,6 +1254,32 @@ DnN5aFtYKiTc19qIC7Nmo+afPdDEf0MlJvEOP5EWl3w=
       self.assertEqual(None, authority.shared_randomness_current_reveal_count)
       self.assertEqual(None, authority.shared_randomness_current_value)
 
+  def test_bandwidth_file_headers(self):
+    """
+    Parses a 'bandwidth-file-headers' line of votes.
+    """
+
+    test_values = {
+      '': {},
+      'timestamp=': {'timestamp': ''},
+      'timestamp=12=34': {'timestamp': '12=34'},
+      'timestamp=123': {'timestamp': '123'},
+      'timestamp=123 version=1.0': {'timestamp': '123', 'version': '1.0'},
+      'timestamp=123 version=1.0 software=neat_thingy': {'timestamp': '123', 'version': '1.0', 'software': 'neat_thingy'},
+    }
+
+    for test_value, expected_value in test_values.items():
+      document = NetworkStatusDocumentV3.create({'vote-status': 'vote', 'bandwidth-file-headers': test_value})
+      self.assertEqual(expected_value, document.bandwidth_file_headers)
+
+    # field must be key=value mappings
+
+    content = NetworkStatusDocumentV3.content({'vote-status': 'vote', 'bandwidth-file-headers': 'key_without_value'})
+    self.assertRaises(ValueError, NetworkStatusDocumentV3, content, True)
+
+    document = NetworkStatusDocumentV3(content, False)
+    self.assertEqual({}, document.bandwidth_file_headers)
+
   def test_with_legacy_directory_authorities(self):
     """
     Includes both normal authorities and those following the '-legacy' format.
