@@ -252,6 +252,13 @@ CONF_CHANGED_EVENT = """650-CONF_CHANGED
 650 OK
 """
 
+CONF_CHANGED_EVENT_MULTIPLE = """650-CONF_CHANGED
+650-ExitPolicy=accept 34.3.4.5
+650-ExitPolicy=accept 3.4.53.3
+650-MaxCircuitDirtiness=20
+650 OK
+"""
+
 # GUARD events from tor v0.2.1.30.
 
 GUARD_NEW = '650 GUARD ENTRY $36B5DBA788246E8369DBAF58577C6BC044A9A374 NEW'
@@ -491,6 +498,7 @@ CIRC_BW_WITH_TIMESTAMP = '650 CIRC_BW ID=11 READ=272 WRITTEN=817 TIME=2012-12-06
 CIRC_BW_BAD_WRITTEN_VALUE = '650 CIRC_BW ID=11 READ=272 WRITTEN=817.7'
 CIRC_BW_BAD_MISSING_ID = '650 CIRC_BW READ=272 WRITTEN=817'
 CIRC_BW_MALFORMED_TIMESTAMP = '650 CIRC_BW ID=11 READ=272 WRITTEN=817 TIME=boom'
+CIRC_BW_WITH_EXTRA_COUNTS = '650 CIRC_BW ID=11 READ=272 WRITTEN=817 TIME=2012-12-06T13:51:11.433755 DELIVERED_READ=12 OVERHEAD_READ=34 DELIVERED_WRITTEN=56 OVERHEAD_WRITTEN=78'
 
 CELL_STATS_1 = '650 CELL_STATS ID=14 \
 OutboundQueue=19403 OutboundConn=15 \
@@ -862,15 +870,35 @@ class TestEvents(unittest.TestCase):
 
   def test_conf_changed(self):
     event = _get_event(CONF_CHANGED_EVENT)
+    self.assertTrue(isinstance(event, stem.response.events.ConfChangedEvent))
 
-    expected_config = {
+    self.assertEqual({
+      'ExitNodes': ['caerSidi'],
+      'MaxCircuitDirtiness': ['20'],
+    }, event.changed)
+
+    self.assertEqual(['ExitPolicy'], event.unset)
+
+    self.assertEqual({
       'ExitNodes': 'caerSidi',
       'MaxCircuitDirtiness': '20',
       'ExitPolicy': None,
-    }
+    }, event.config)
 
+    event = _get_event(CONF_CHANGED_EVENT_MULTIPLE)
     self.assertTrue(isinstance(event, stem.response.events.ConfChangedEvent))
-    self.assertEqual(expected_config, event.config)
+
+    self.assertEqual({
+      'ExitPolicy': ['accept 34.3.4.5', 'accept 3.4.53.3'],
+      'MaxCircuitDirtiness': ['20'],
+    }, event.changed)
+
+    self.assertEqual([], event.unset)
+
+    self.assertEqual({
+      'ExitPolicy': 'accept 3.4.53.3',  # overwrote with second value
+      'MaxCircuitDirtiness': '20',
+    }, event.config)
 
   def test_descchanged_event(self):
     # all we can check for is that the event is properly parsed as a
@@ -1524,6 +1552,16 @@ class TestEvents(unittest.TestCase):
     self.assertEqual(817, event.written)
     self.assertEqual(datetime.datetime(2012, 12, 6, 13, 51, 11, 433755), event.time)
 
+    event = _get_event(CIRC_BW_WITH_EXTRA_COUNTS)
+    self.assertEqual('11', event.id)
+    self.assertEqual(272, event.read)
+    self.assertEqual(817, event.written)
+    self.assertEqual(datetime.datetime(2012, 12, 6, 13, 51, 11, 433755), event.time)
+    self.assertEqual(12, event.delivered_read)
+    self.assertEqual(56, event.delivered_written)
+    self.assertEqual(34, event.overhead_read)
+    self.assertEqual(78, event.overhead_written)
+
     self.assertRaises(ProtocolError, _get_event, CIRC_BW_BAD_WRITTEN_VALUE)
     self.assertRaises(ProtocolError, _get_event, CIRC_BW_BAD_MISSING_ID)
     self.assertRaises(ProtocolError, _get_event, CIRC_BW_MALFORMED_TIMESTAMP)
