@@ -17,11 +17,10 @@ import stem.prereq
 
 from stem.control import Controller
 from stem.descriptor.networkstatus import NetworkStatusDocumentV3
-from stem.descriptor.remote import DIRECTORY_AUTHORITIES
 from stem.descriptor.router_status_entry import RouterStatusEntryV3
 from stem.descriptor.server_descriptor import RelayDescriptor
+from stem.directory import DIRECTORY_AUTHORITIES
 from stem.response import ControlMessage
-from stem.util import str_type
 
 from test.unit import exec_documentation_example
 
@@ -39,7 +38,7 @@ PURPOSE=%s'
 
 PATH_CONTENT = '$%s=%s,$%s=%s,$%s=%s'
 
-LIST_CIRCUITS_OUTPUT = str_type("""\
+LIST_CIRCUITS_OUTPUT = """\
 
 Circuit 4 (GENERAL)
  |- B1FA7D51B8B6F0CB585D944F450E7C06EDE7E44C (ByTORAndTheSnowDog, 173.209.180.61)
@@ -55,9 +54,9 @@ Circuit 10 (GENERAL)
  |- B1FA7D51B8B6F0CB585D944F450E7C06EDE7E44C (ByTORAndTheSnowDog, 173.209.180.61)
  |- 00C2C2A16AEDB51D5E5FB7D6168FC66B343D822F (ph3x, 86.59.119.83)
  +- 65242C91BFF30F165DA4D132C81A9EBA94B71D62 (torexit16, 176.67.169.171)
-""")
+"""
 
-EXIT_USED_OUTPUT = str_type("""\
+EXIT_USED_OUTPUT = """\
 Tracking requests for tor exits. Press 'enter' to end.
 
 Exit relay for our connection to 64.15.112.44:80
@@ -66,15 +65,15 @@ Exit relay for our connection to 64.15.112.44:80
   nickname: chaoscomputerclub19
   locale: unknown
 
-""")
+"""
 
-OUTDATED_RELAYS_OUTPUT = str_type("""\
+OUTDATED_RELAYS_OUTPUT = """\
 Checking for outdated relays...
 
   0.1.0           Sambuddha Basu
 
 2 outdated relays found, 1 had contact information
-""")
+"""
 
 COMPARE_FLAGS_OUTPUT = """\
 maatuska has the Running flag but moria1 doesn't: E2BB13AA2F6960CD93ABE5257A825687F3973C62
@@ -234,8 +233,8 @@ class TestTutorialExamples(unittest.TestCase):
 
   @patch('sys.stdout', new_callable = StringIO)
   @patch('stem.descriptor.remote.Query')
-  @patch('stem.descriptor.remote.get_authorities')
-  def test_compare_flags(self, get_authorities_mock, query_mock, stdout_mock):
+  @patch('stem.directory.Authority.from_cache')
+  def test_compare_flags(self, authorities_mock, query_mock, stdout_mock):
     if stem.prereq._is_python_26():
       # example imports OrderedDict from collections which doesn't work under
       # python 2.6
@@ -243,7 +242,7 @@ class TestTutorialExamples(unittest.TestCase):
       self.skipTest("(example doesn't support python 2.6)")
       return
 
-    get_authorities_mock().items.return_value = [('moria1', DIRECTORY_AUTHORITIES['moria1']), ('maatuska', DIRECTORY_AUTHORITIES['maatuska'])]
+    authorities_mock().items.return_value = [('moria1', DIRECTORY_AUTHORITIES['moria1']), ('maatuska', DIRECTORY_AUTHORITIES['maatuska'])]
 
     fingerprint = [
       ('92FCB6748A40E6088E22FBAB943AB2DD743EA818', 'kvy2dIpA5giOIvurlDqy3XQ+qBg='),
@@ -281,9 +280,9 @@ class TestTutorialExamples(unittest.TestCase):
     self.assert_equal_unordered(COMPARE_FLAGS_OUTPUT, stdout_mock.getvalue())
 
   @patch('sys.stdout', new_callable = StringIO)
-  @patch('stem.descriptor.remote.get_authorities')
+  @patch('stem.directory.Authority.from_cache')
   @patch('stem.descriptor.remote.DescriptorDownloader.query')
-  def test_votes_by_bandwidth_authorities(self, query_mock, get_authorities_mock, stdout_mock):
+  def test_votes_by_bandwidth_authorities(self, query_mock, authorities_mock, stdout_mock):
     directory_values = [
       DIRECTORY_AUTHORITIES['gabelmoo'],
       DIRECTORY_AUTHORITIES['moria1'],
@@ -291,7 +290,7 @@ class TestTutorialExamples(unittest.TestCase):
     ]
 
     directory_values[0].address = '131.188.40.189'
-    get_authorities_mock().values.return_value = directory_values
+    authorities_mock().values.return_value = directory_values
 
     entry_with_measurement = RouterStatusEntryV3.create({'w': 'Bandwidth=1 Measured=1'})
     entry_without_measurement = RouterStatusEntryV3.create()
